/* ----------------------------------------------------------------------------
 * Raima Database Manager
 *
 * Copyright (c) 2010 Raima Inc.,  All rights reserved.
 *
 * Use of this software, whether in source code format, or in executable,
 * binary object code form, is governed by the Raima LICENSE which
 * is fully described in the LICENSE.TXT file, included within this
 * distribution of files.
 * ----------------------------------------------------------------------------
 */

/** \file rdmtimestamptzapi.h
 *  \brief Header for the public timestamp with timezone API
 */

#include "rdmdatetimetypes.h"
#include "rdmretcodetypes.h"

#if defined(RDM_UNICODE)
#define rdm_timestamptzFromString RDM_T_FUNC (rdm_timestamptzFromString)
#define rdm_timestamptzNowAsString RDM_T_FUNC (rdm_timestamptzNowAsString)
#define rdm_timestamptzToString RDM_T_FUNC (rdm_timestamptzToString)
#endif

/** \addtogroup timestamp
 * @{
 */

#if defined(__cplusplus)
extern "C" {
#endif

/** \brief Convert a local timestamp to timestamp with timezone
 *
 *  Convert a local timestamp to timestamp with timezone.
 *
 * \retval{sOKAY}
 * \retval{eBASE_INVTIMEZONE}
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_timestamptzFromTimestamp (
    RDM_PACKED_TIMESTAMP_T tsVal, /**< [in]  local timestamp value */
    int16_t tz,                   /**< [in]  timezone offset (in minutes) */
    RDM_PACKED_TIMESTAMPTZ_T
        *pTszVal /**< [out] resulting timestamp with timezone value */
);

/** \brief Convert from a string to a timestamp with timezone
 *
 *  Convert a string to a packed timestamp with timezone.
 *
 *  \retval{sOKAY}
 *  \retval{eBASE_INVDATE}
 *  \retval{eINVDATEFMT}
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_timestamptzFromString (
    const RDM_TCHAR_T *str,        /**< [in]  timestamp value in string */
    RDM_DATE_FORMAT date_fmt,      /**< [in]  RDM date format */
    RDM_PACKED_TIMESTAMPTZ_T *ptsz /**< [out] result timestamp value */
);

/** \brief Get the current timestamp with timezone
 *
 *  Get the current timestamp with timezone.
 *
 *  \retval{sOKAY}
 *  \retval{eBASE_INVTIMEZONE}
 *
 *  \see rdm_timestampNowAsString
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_timestamptzNow (
    int16_t time_zone,            /**< [in]  Timezone value (in min) */
    RDM_PACKED_TIMESTAMPTZ_T *tsz /**< [out] current timestamp with timezone */
);

/** \brief Get the current timestamp with timezone as string
 *
 *  Get the current timestamp with timezone.
 *
 *  \retval{sOKAY}
 *  \retval{eBASE_INVTIMEZONE}
 *  \retval{sTRUNCATE}
 *
 *  \see rdm_dateToday
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_timestamptzNowAsString (
    int16_t time_zone,   /**< [in]  Timezone value (in min) */
    RDM_TCHAR_T *tszbuf, /**< [out] NOW value in string */
    size_t buflen /**< [in]  Size of tsbuf in number of ASCII characters it can hold */
);

/** \brief Convert a timestamp with timezone to a string
 *
 *  Convert a packed timestamp with timezone to local timestamp.
 *
 *
 *  \retval{sOKAY}
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_timestamptzToTimestamp (
    RDM_PACKED_TIMESTAMPTZ_T tstzVal, /**< [in]  timestamp with timezone value */
    int16_t tz_disp,                  /**< [in]  local timezone */
    RDM_PACKED_TIMESTAMP_T *pTsVal /**< [out] resulting local timestamp value */
);

/** \brief Convert a timestamp with timezone to a string
 *
 *  Convert a packed timestamp with timezone to a string.
 *
 *  \retval{sOKAY}
 *  \retval{eBASE_INVDATE}
 *  \retval{eINVDATESEP}
 *  \retval{sTRUNCATE}
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_timestamptzToString (
    RDM_PACKED_TIMESTAMPTZ_T tszVal, /**< [in] RDM timestamp with timezone value */
    RDM_DATE_FORMAT date_format,     /**< [in]  RDM date format */
    RDM_TCHAR_T date_sep,            /**< [in]  date separator */
    RDM_TIME_FORMAT time_format,     /**< [in]  RDM time format */
    RDM_TCHAR_T *buf, /**< [out] result timestamp with timezone string */
    size_t bufSize, /**< [in]  buffer size in number of ASCII characters it can hold */
    size_t *puSize /**< [out] resulting size in number of characters */
);

/** \brief Get the timestamp of the epoch
 *
 *  Get the timestamp with timezone of the epoch.
 *
 *  \return The timestamp of the epoch
 */
RDM_EXPORT RDM_PACKED_TIMESTAMPTZ_T EXTERNAL_FCN rdm_timestamptzZero (void);

#if defined(__cplusplus)
}
#endif

/** @}*/
